@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Process do
   @moduledoc """
   Conveniences for working with processes and the process dictionary.
@@ -84,10 +88,13 @@ defmodule Process do
   Tells whether the given process is alive on the local node.
 
   If the process identified by `pid` is alive (that is, it's not exiting and has
-  not exited yet) than this function returns `true`. Otherwise, it returns
+  not exited yet) then this function returns `true`. Otherwise, it returns
   `false`.
 
   `pid` must refer to a process running on the local node or `ArgumentError` is raised.
+  To check whether a process on any node is alive you can use the [`:erpc`](`:erpc`) module.
+
+      :erpc.call(node(pid), Process, :alive?, [pid])
 
   Inlined by the compiler.
   """
@@ -216,6 +223,16 @@ defmodule Process do
 
   Inlined by the compiler.
 
+  > #### Differences to `Kernel.exit/1` {: .info }
+  >
+  > The functions `Kernel.exit/1` and `Process.exit/2` are
+  > named similarly but provide very different functionalities. The
+  > `Kernel:exit/1` function should be used when the intent is to stop the current
+  > process while `Process.exit/2` should be used when the intent is to send an
+  > exit signal to another process. Note also that `Kernel.exit/1` can be caught
+  > with `try/1` while `Process.exit/2` can only be handled by trapping exits and
+  > when the signal is different than `:kill`.
+
   ## Examples
 
       Process.exit(pid, :kill)
@@ -233,6 +250,12 @@ defmodule Process do
   the current process will sleep forever, and not
   consume or reply to messages.
 
+  > #### Sleeping limit {: .info }
+  >
+  > Before Elixir v1.18, `sleep/1` did not accept integer timeout values greater
+  > than `16#ffffffff`, that is, `2^32-1`. Since Elixir v1.18, arbitrarily-high integer
+  > values are accepted.
+
   **Use this function with extreme care**. For almost all situations
   where you would use `sleep/1` in Elixir, there is likely a
   more correct, faster and precise way of achieving the same with
@@ -299,7 +322,15 @@ defmodule Process do
       end
 
   """
+
+  # Max value for a receive's after clause
+  @max_receive_after 0xFFFFFFFF
+
   @spec sleep(timeout) :: :ok
+  def sleep(timeout) when is_integer(timeout) and timeout > @max_receive_after do
+    receive after: (@max_receive_after -> sleep(timeout - @max_receive_after))
+  end
+
   def sleep(timeout)
       when is_integer(timeout) and timeout >= 0
       when timeout == :infinity do
@@ -552,10 +583,15 @@ defmodule Process do
 
       send(pid, {:ping, ref_and_alias})
 
-      receive do: msg -> msg
+      receive do: (msg -> msg)
       #=> :pong
 
-      receive do: msg -> msg
+      ref_and_alias = Process.monitor(pid, alias: :reply_demonitor)
+      #=> #Reference<0.906660723.3006791681.40191>
+
+      send(pid, {:ping, ref_and_alias})
+
+      receive do: (msg -> msg)
       #=> {:DOWN, #Reference<0.906660723.3006791681.40191>, :process, #PID<0.118.0>, :noproc}
 
   """
