@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Task.Supervised do
   @moduledoc false
   @ref_timeout 5000
@@ -209,15 +213,15 @@ defmodule Task.Supervised do
     ordered = Keyword.get(options, :ordered, true)
     zip_input_on_exit = Keyword.get(options, :zip_input_on_exit, false)
 
-    unless is_integer(max_concurrency) and max_concurrency > 0 do
+    if not (is_integer(max_concurrency) and max_concurrency > 0) do
       raise ArgumentError, ":max_concurrency must be an integer greater than zero"
     end
 
-    unless on_timeout in [:exit, :kill_task] do
+    if on_timeout not in [:exit, :kill_task] do
       raise ArgumentError, ":on_timeout must be either :exit or :kill_task"
     end
 
-    unless (is_integer(timeout) and timeout >= 0) or timeout == :infinity do
+    if not ((is_integer(timeout) and timeout >= 0) or timeout == :infinity) do
       raise ArgumentError, ":timeout must be either a positive integer or :infinity"
     end
 
