@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule Version do
   @moduledoc ~S"""
   Functions for parsing and matching versions against requirements.
@@ -114,6 +118,8 @@ defmodule Version do
   @type build :: String.t() | nil
   @type t :: %__MODULE__{major: major, minor: minor, patch: patch, pre: pre, build: build}
 
+  @type match_opts :: [allow_pre: boolean()]
+
   defmodule Requirement do
     @moduledoc """
     A struct that holds version requirement information.
@@ -217,6 +223,12 @@ defmodule Version do
   end
 
   defmodule InvalidRequirementError do
+    @moduledoc """
+    An exception raised when a version requirement is invalid.
+
+    For example, see `Version.parse_requirement!/1`.
+    """
+
     defexception [:requirement]
 
     @impl true
@@ -231,6 +243,12 @@ defmodule Version do
   end
 
   defmodule InvalidVersionError do
+    @moduledoc """
+    An exception raised when a version is invalid.
+
+    For example, see `Version.parse!/1`.
+    """
+
     defexception [:version]
 
     @impl true
@@ -280,7 +298,7 @@ defmodule Version do
       ** (Version.InvalidRequirementError) invalid requirement: "== == 1.0.0"
 
   """
-  @spec match?(version, requirement, keyword) :: boolean
+  @spec match?(version, requirement, match_opts) :: boolean
   def match?(version, requirement, opts \\ [])
 
   def match?(version, requirement, opts) when is_binary(requirement) do
@@ -424,7 +442,7 @@ defmodule Version do
 
   If `string` is an invalid requirement, a `Version.InvalidRequirementError` is raised.
 
-  # Examples
+  ## Examples
 
       iex> Version.parse_requirement!("== 2.0.1")
       Version.parse_requirement!("== 2.0.1")
@@ -467,7 +485,7 @@ defmodule Version do
   @doc """
   Converts the given version to a string.
 
-  ### Examples
+  ## Examples
 
       iex> Version.to_string(%Version{major: 1, minor: 2, patch: 3})
       "1.2.3"
@@ -674,7 +692,7 @@ end
 
 defimpl Inspect, for: Version.Requirement do
   def inspect(%Version.Requirement{source: source}, opts) do
-    colorized = Inspect.Algebra.color("\"" <> source <> "\"", :string, opts)
+    colorized = Inspect.Algebra.color_doc("\"" <> source <> "\"", :string, opts)
 
     Inspect.Algebra.concat(["Version.parse_requirement!(", colorized, ")"])
   end
