@@ -1,4 +1,11 @@
+%% SPDX-License-Identifier: Apache-2.0
+%% SPDX-FileCopyrightText: 2021 The Elixir Team
+%% SPDX-FileCopyrightText: 2012 Plataformatec
+
 -module(elixir_quote).
+
+-feature(maybe_expr, enable).
+
 -export([escape/3, linify/3, linify_with_context_counter/3, build/7, quote/2, has_unquotes/1, fun_to_quoted/1]).
 -export([dot/5, tail_list/3, list/2, validate_runtime/2, shallow_validate_ast/1]). %% Quote callbacks
 
@@ -11,7 +18,7 @@
   line=false,
   file=nil,
   context=nil,
-  op=none, % none | prune_metadata | add_context
+  op=escape, % escape | escape_and_prune | {struct, Module} | quote
   aliases_hygiene=nil,
   imports_hygiene=nil,
   unquote=true,
@@ -133,19 +140,31 @@ do_tuple_linify(Fun, Meta, Left, Right, Var) ->
 %% Escapes the given expression. It is similar to quote, but
 %% lines are kept and hygiene mechanisms are disabled.
 escape(Expr, Op, Unquote) ->
-  do_quote(Expr, #elixir_quote{
-    line=true,
-    file=nil,
-    op=Op,
-    unquote=Unquote
-  }).
+  try
+    do_quote(Expr, #elixir_quote{
+      line=true,
+      file=nil,
+      op=Op,
+      unquote=Unquote
+    })
+  catch
+    Kind:Reason:Stacktrace ->
+      Pruned = lists:dropwhile(fun
+        ({?MODULE, _, _, _}) -> true;
+        (_) -> false
+      end, Stacktrace),
+      erlang:raise(Kind, Reason, Pruned)
+  end.
 
-do_escape({Left, Meta, Right}, #elixir_quote{op=prune_metadata} = Q) when is_list(Meta) ->
+do_escape({Left, Meta, Right}, #elixir_quote{op=escape_and_prune} = Q) when is_list(Meta) ->
   TM = [{K, V} || {K, V} <- Meta, (K == no_parens) orelse (K == line) orelse (K == delimiter)],
   TL = do_quote(Left, Q),
   TR = do_quote(Right, Q),
   {'{}', [], [TL, TM, TR]};
 
+do_escape({Left, Right}, Q) ->
+  {do_quote(Left, Q), do_quote(Right, Q)};
+
 do_escape(Tuple, Q) when is_tuple(Tuple) ->
   TT = do_quote(tuple_to_list(Tuple), Q),
   {'{}', [], TT};
@@ -160,22 +179,42 @@ do_escape(BitString, _) when is_bitstring(BitString) ->
   end;
 
 do_escape(Map, Q) when is_map(Map) ->
-  TT =
-    [if
-      is_reference(V) ->
-        argument_error(<<('Elixir.Kernel':inspect(Map, []))/binary, " contains a reference (",
-                         ('Elixir.Kernel':inspect(V, []))/binary, ") and therefore it cannot be escaped ",
-                         "(it must be defined within a function instead). ", (bad_escape_hint())/binary>>);
-      true ->
+  maybe
+    #{'__struct__' := Module} ?= Map,
+    true ?= is_atom(Module),
+    % We never escape ourselves (it can only happen during Elixir bootstrapping)
+    true ?= (Q#elixir_quote.op /= {struct, Module}),
+    {module, Module} ?= code:ensure_loaded(Module),
+    true ?= erlang:function_exported(Module, '__escape__', 1),
+    case Q#elixir_quote.op of
+      {struct, _Module} ->
+        argument_error(<<('Elixir.Kernel':inspect(Module))/binary,
+          " defines custom escaping rules which are not supported in struct defaults",
+          (bad_escape_hint())/binary>>);
+
+      _ ->
+        Expr = Module:'__escape__'(Map),
+        case shallow_valid_ast(Expr) of
+          true -> Expr;
+          false -> argument_error(
+            <<('Elixir.Kernel':inspect(Module))/binary, ".__escape__/1 returned invalid AST: ", ('Elixir.Kernel':inspect(Expr))/binary>>
+          )
+        end
+    end
+  else
+    _ ->
+      TT = [
         {do_quote(K, Q), do_quote(V, Q)}
-     end || {K, V} <- lists:sort(maps:to_list(Map))],
-  {'%{}', [], TT};
+        || {K, V} <- lists:sort(maps:to_list(Map))
+      ],
+      {'%{}', [], TT}
+  end;
 
 do_escape([], _) ->
   [];
 
 do_escape([H | T], #elixir_quote{unquote=false} = Q) ->
-  do_quote_simple_list(T, do_quote(H, Q), Q);
+  do_quote_simple_list(T, do_escape(H, Q), Q);
 
 do_escape([H | T], Q) ->
   %% The improper case is inefficient, but improper lists are rare.
@@ -203,11 +242,11 @@ do_escape(Other, _) ->
   bad_escape(Other).
 
 bad_escape(Arg) ->
-  argument_error(<<"cannot escape ", ('Elixir.Kernel':inspect(Arg, []))/binary, ". ",
+  argument_error(<<"cannot escape ", ('Elixir.Kernel':inspect(Arg, []))/binary,
                    (bad_escape_hint())/binary>>).
 
 bad_escape_hint() ->
-  <<"The supported values are: lists, tuples, maps, atoms, numbers, bitstrings, ",
+  <<". The supported values are: lists, tuples, maps, atoms, numbers, bitstrings, ",
     "PIDs and remote functions in the format &Mod.fun/arity">>.
 
 %% Quote entry points
@@ -221,7 +260,7 @@ build(Meta, Line, File, Context, Unquote, Generated, E) ->
   validate_runtime(generated, Generated),
 
   Q = #elixir_quote{
-    op=add_context,
+    op=quote,
     aliases_hygiene=E,
     imports_hygiene=E,
     line=VLine,
@@ -301,7 +340,7 @@ do_quote({quote, Meta, [Arg]}, Q) when is_list(Meta) ->
   TArg = do_quote(Arg, Q#elixir_quote{unquote=false}),
 
   NewMeta = case Q of
-    #elixir_quote{op=add_context, context=Context} -> keystore(context, Meta, Context);
+    #elixir_quote{op=quote, context=Context} -> keystore(context, Meta, Context);
     _ -> Meta
   end,
 
@@ -312,13 +351,12 @@ do_quote({quote, Meta, [Opts, Arg]}, Q) when is_list(Meta) ->
   TArg = do_quote(Arg, Q#elixir_quote{unquote=false}),
 
   NewMeta = case Q of
-    #elixir_quote{op=add_context, context=Context} -> keystore(context, Meta, Context);
+    #elixir_quote{op=quote, context=Context} -> keystore(context, Meta, Context);
     _ -> Meta
   end,
 
   {'{}', [], [quote, meta(NewMeta, Q), [TOpts, TArg]]};
 
-%
 do_quote({unquote, Meta, [Expr]}, #elixir_quote{unquote=true, shallow_validate=Validate}) when is_list(Meta) ->
   case Validate of
     true -> {{'.', Meta, [?MODULE, shallow_validate_ast]}, Meta, [Expr]};
@@ -339,7 +377,7 @@ do_quote({'__aliases__', Meta, [H | T]}, #elixir_quote{aliases_hygiene=(#{}=E)}
 
 %% Vars
 
-do_quote({Name, Meta, nil}, #elixir_quote{op=add_context} = Q)
+do_quote({Name, Meta, nil}, #elixir_quote{op=quote} = Q)
     when is_atom(Name), is_list(Meta) ->
   ImportMeta = case Q#elixir_quote.imports_hygiene of
     nil -> Meta;
@@ -395,7 +433,7 @@ do_quote({Left, Right}, Q) ->
 
 %% Everything else
 
-do_quote(Other, #elixir_quote{op=Op} = Q) when Op =/= add_context ->
+do_quote(Other, #elixir_quote{op=Op} = Q) when Op =/= quote ->
   do_escape(Other, Q);
 
 do_quote({_, _, _} = Tuple, Q) ->
@@ -555,8 +593,10 @@ argument_error(Message) ->
 
 %% Helpers
 
+meta(Meta, #elixir_quote{op=quote} = Q) ->
+  generated(keep(keydelete(column, Meta), Q), Q);
 meta(Meta, Q) ->
-  generated(keep(keydelete(column, Meta), Q), Q).
+  do_quote(Meta, Q).
 
 generated(Meta, #elixir_quote{generated=true}) -> [{generated, true} | Meta];
 generated(Meta, #elixir_quote{generated=false}) -> Meta.
