@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("../test_helper.exs", __DIR__)
 
 defmodule Kernel.ComprehensionTest do
@@ -147,7 +151,7 @@ defmodule Kernel.ComprehensionTest do
 
   test "for comprehensions with errors on filters" do
     assert_raise ArgumentError, fn ->
-      for x <- 1..3, hd(x), do: x * 2
+      for x <- 1..3, hd(x), do: :ok
     end
   end
 
@@ -293,7 +297,7 @@ defmodule Kernel.ComprehensionTest do
         acc -> Map.update(acc, x, [y], &[y | &1])
       end
 
-    assert acc == %{1 => 'olleh', 3 => 'olleh'}
+    assert acc == %{1 => ~c"olleh", 3 => ~c"olleh"}
   end
 
   test "for comprehensions with matched reduce" do
@@ -338,7 +342,7 @@ defmodule Kernel.ComprehensionTest do
 
   test "list for comprehensions with errors on filters" do
     assert_raise ArgumentError, fn ->
-      for x <- [1, 2, 3], hd(x), do: x * 2
+      for x <- [1, 2, 3], hd(Process.get(:unused, x)), do: x * 2
     end
   end
 
@@ -401,7 +405,7 @@ defmodule Kernel.ComprehensionTest do
         acc -> Map.update(acc, x, [y], &[y | &1])
       end
 
-    assert acc == %{1 => 'olleh', 3 => 'olleh'}
+    assert acc == %{1 => ~c"olleh", 3 => ~c"olleh"}
   end
 
   ## Binary generators (inlined by the compiler)
@@ -496,7 +500,11 @@ defmodule Kernel.ComprehensionTest do
   test "binary for comprehensions with variable size" do
     s = 16
     bin = <<1, 2, 3, 4, 5, 6>>
-    assert for(<<x::size(s) <- bin>>, into: "", do: to_bin(div(x, 2))) == <<129, 130, 131>>
+    assert for(<<x::size(^s) <- bin>>, into: "", do: to_bin(div(x, 2))) == <<129, 130, 131>>
+
+    s = 8
+    bin = <<1, 2, 3, 4, 5, 6>>
+    assert for(<<s, x::size(^s) <- bin>>, into: "", do: <<x * s>>) == <<2, 12, 30>>
 
     # Aligned
     bin = <<8, 1, 16, 2, 3>>
@@ -526,6 +534,6 @@ defmodule Kernel.ComprehensionTest do
         acc -> Map.update(acc, x, [y], &[y | &1])
       end
 
-    assert acc == %{1 => 'olleh', 3 => 'olleh'}
+    assert acc == %{1 => ~c"olleh", 3 => ~c"olleh"}
   end
 end
