@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("test_helper.exs", __DIR__)
 
 defmodule SystemTest do
@@ -128,7 +132,7 @@ defmodule SystemTest do
         # There is a bug in OTP where find_executable is finding
         # entries on the current directory. If this is the case,
         # we should avoid the assertion below.
-        unless System.find_executable(@echo) do
+        if !System.find_executable(@echo) do
           assert :enoent = catch_error(System.cmd(@echo, ~w[/c echo hello]))
         end
 
@@ -212,7 +216,7 @@ defmodule SystemTest do
         # There is a bug in OTP where find_executable is finding
         # entries on the current directory. If this is the case,
         # we should avoid the assertion below.
-        unless System.find_executable(@echo) do
+        if !System.find_executable(@echo) do
           assert :enoent = catch_error(System.cmd(@echo, ["hello"]))
         end
 
