@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule ExUnit do
   @moduledoc """
   Unit testing framework for Elixir.
@@ -11,11 +15,11 @@ defmodule ExUnit do
       # 1) Start ExUnit.
       ExUnit.start()
 
-      # 2) Create a new test module (test case) and use "ExUnit.Case".
+      # 2) Create a new test module and use "ExUnit.Case".
       defmodule AssertionTest do
-        # 3) Note that we pass "async: true", this runs the test case
-        #    concurrently with other test cases. The individual tests
-        #    within each test case are still run serially.
+        # 3) Note that we pass "async: true", this runs the tests in the
+        #    test module concurrently with other test modules. The
+        #    individual tests within each test module are still run serially.
         use ExUnit.Case, async: true
 
         # 4) Use the "test" macro instead of "def" for clarity.
@@ -89,6 +93,50 @@ defmodule ExUnit do
 
   @type test_id :: {module, name :: atom}
 
+  @typedoc """
+  Configuration options for ExUnit.
+
+  See `configure/1` for detailed documentation of each option.
+  """
+  @type configure_opts :: [
+          {:assert_receive_timeout, non_neg_integer()}
+          | {:autorun, boolean()}
+          | {:capture_log, boolean() | [level: Logger.level()]}
+          | {:colors,
+             [
+               enabled: boolean(),
+               success: atom(),
+               invalid: atom(),
+               skipped: atom(),
+               failure: atom(),
+               error_info: atom(),
+               extra_info: atom(),
+               location_info: [atom()],
+               diff_insert: atom(),
+               diff_insert_whitespace: IO.ANSI.ansidata(),
+               diff_delete: atom(),
+               diff_delete_whitespace: IO.ANSI.ansidata()
+             ]}
+          | {:exclude, keyword()}
+          | {:exit_status, non_neg_integer()}
+          | {:failures_manifest_path, String.t()}
+          | {:formatters, [module()]}
+          | {:include, keyword()}
+          | {:max_cases, pos_integer()}
+          | {:max_failures, pos_integer() | :infinity}
+          | {:only_test_ids, [test_id()]}
+          | {:rand_algorithm, atom()}
+          | {:refute_receive_timeout, non_neg_integer()}
+          | {:seed, non_neg_integer()}
+          | {:slowest, non_neg_integer()}
+          | {:slowest_modules, non_neg_integer()}
+          | {:stacktrace_depth, non_neg_integer()}
+          | {:timeout, pos_integer()}
+          | {:trace, boolean()}
+          | {:test_location_relative_path, String.t()}
+          | {atom(), term()}
+        ]
+
   defmodule Test do
     @moduledoc """
     A struct that keeps information about the test.
@@ -101,9 +149,10 @@ defmodule ExUnit do
       * `:time` - the duration in microseconds of the test's runtime
       * `:tags` - the test tags
       * `:logs` - the captured logs
+      * `:parameters` - the test parameters
 
     """
-    defstruct [:name, :case, :module, :state, time: 0, tags: %{}, logs: ""]
+    defstruct [:name, :case, :module, :state, time: 0, tags: %{}, logs: "", parameters: %{}]
 
     # TODO: Remove the `:case` field on v2.0
     @type t :: %__MODULE__{
@@ -113,7 +162,8 @@ defmodule ExUnit do
             state: ExUnit.state(),
             time: non_neg_integer,
             tags: map,
-            logs: String.t()
+            logs: String.t(),
+            parameters: map
           }
   end
 
@@ -123,20 +173,28 @@ defmodule ExUnit do
 
     It is received by formatters and contains the following fields:
 
-      * `:file`  - (since v1.11.0) the file of the test module
+      * `:file` - (since v1.11.0) the file of the test module
 
-      * `:name`  - the test module name
+      * `:name` - the test module name
+
+      * `:parameters` - (since v1.18.0) the test module parameters
+
+      * `:setup_all?` - (since v1.18.0) if the test module requires a setup all
 
       * `:state` - the test error state (see `t:ExUnit.state/0`)
 
+      * `:tags` - all tags in this module
+
       * `:tests` - all tests in this module
 
     """
-    defstruct [:file, :name, :state, tags: %{}, tests: []]
+    defstruct [:file, :name, :setup_all?, :state, parameters: %{}, tags: %{}, tests: []]
 
     @type t :: %__MODULE__{
             file: binary(),
             name: module,
+            parameters: map(),
+            setup_all?: boolean(),
             state: ExUnit.state(),
             tags: map,
             tests: [ExUnit.Test.t()]
@@ -204,7 +262,7 @@ defmodule ExUnit do
   If you want to run tests manually, you can set the `:autorun` option
   to `false` and use `run/0` to run tests.
   """
-  @spec start(Keyword.t()) :: :ok
+  @spec start(configure_opts()) :: :ok
   def start(options \\ []) do
     {:ok, _} = Application.ensure_all_started(:ex_unit)
 
@@ -273,7 +331,7 @@ defmodule ExUnit do
       filter. See the "Filters" section in the documentation for `ExUnit.Case`;
 
     * `:exit_status` - specifies an alternate exit status to use when the test
-      suite fails. Defaults to 2;
+      suite fails. Defaults to `2`;
 
     * `:failures_manifest_path` - specifies a path to the file used to store failures
       between runs;
@@ -302,8 +360,8 @@ defmodule ExUnit do
 
     * `:rand_algorithm` - algorithm to be used when generating the test seed.
       Available algorithms can be found in Erlang's
-      [`:rand`](https://www.erlang.org/doc/man/rand.html) documentation (see
-      [`:rand.builting_arg/0`](https://www.erlang.org/doc/man/rand.html#type-builtin_alg)).
+      [`:rand`](`:rand`) documentation (see
+      [`:rand.builting_arg/0`](https://www.erlang.org/doc/apps/stdlib/rand.html#t:builtin_alg/0)).
       Available since v1.16.0. Before v1.16.0, the algorithm was hard-coded to
       `:exs1024`. On Elixir v1.16.0 and after, the default changed to `:exsss`;
 
@@ -312,8 +370,8 @@ defmodule ExUnit do
 
     * `:seed` - an integer seed value to randomize the test suite. This seed
       is also mixed with the test module and name to create a new unique seed
-      on every test, which is automatically fed into the `:rand` module. This
-      provides randomness between tests, but predictable and reproducible
+      on every test, which is automatically fed into the [`:rand`](`:rand`) module.
+      This provides randomness between tests, but predictable and reproducible
       results. A `:seed` of `0` will disable randomization and the tests in each
       file will always run in the order that they were defined in;
 
@@ -343,7 +401,7 @@ defmodule ExUnit do
   and these options can then be used in places such as custom formatters. These
   other options will be ignored by ExUnit itself.
   """
-  @spec configure(Keyword.t()) :: :ok
+  @spec configure(configure_opts()) :: :ok
   def configure(options) when is_list(options) do
     Enum.each(options, fn {k, v} ->
       Application.put_env(:ex_unit, k, v)
@@ -402,12 +460,10 @@ defmodule ExUnit do
   @spec run([module()]) :: suite_result()
   def run(additional_modules \\ []) do
     for module <- additional_modules do
-      module_attributes = module.__info__(:attributes)
-
-      if true in Keyword.get(module_attributes, :ex_unit_async, []) do
-        ExUnit.Server.add_async_module(module)
+      if Code.ensure_loaded?(module) and function_exported?(module, :__ex_unit__, 1) do
+        ExUnit.Server.add_module(module, module.__ex_unit__(:config))
       else
-        ExUnit.Server.add_sync_module(module)
+        raise(ArgumentError, "#{inspect(module)} is not a ExUnit.Case module")
       end
     end
 
