@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 defmodule ExUnit.Diff do
   @moduledoc false
 
@@ -218,19 +222,19 @@ defmodule ExUnit.Diff do
   # Guards
 
   defp diff_guard({:when, _, [expression, clause]}, right, env) do
-    {diff_expression, post_env} = diff_quoted(expression, right, nil, env)
+    {diff, post_env} = diff_quoted(expression, right, nil, env)
 
     {guard_clause, guard_equivalent?} =
-      if diff_expression.equivalent? do
+      if diff.equivalent? do
         bindings = Map.merge(post_env.pins, post_env.current_vars)
         diff_guard_clause(clause, bindings)
       else
         {clause, false}
       end
 
-    diff = %__MODULE__{
-      diff_expression
-      | left: {:when, [], [diff_expression.left, guard_clause]},
+    diff = %{
+      diff
+      | left: {:when, [], [diff.left, guard_clause]},
         equivalent?: guard_equivalent?
     }
 
@@ -373,7 +377,9 @@ defmodule ExUnit.Diff do
     {parsed_left, improper_left, operators_left, length_left} =
       split_left_list(left, 0, env.context)
 
-    {parsed_right, improper_right} = split_right_list(right, length_left, [])
+    element_limit = if improper_left == [], do: -1, else: length_left
+    {parsed_right, improper_right} = split_right_list(right, element_limit, [])
+
     {parsed_diff, parsed_post_env} = myers_difference_list(parsed_left, parsed_right, env)
 
     {improper_diff, improper_post_env} =
@@ -406,7 +412,7 @@ defmodule ExUnit.Diff do
     diff(left, right, env)
   end
 
-  defp split_right_list([head | tail], length, acc) when length > 0,
+  defp split_right_list([head | tail], length, acc) when length != 0,
     do: split_right_list(tail, length - 1, [head | acc])
 
   defp split_right_list(rest, _length, acc),
@@ -825,10 +831,10 @@ defmodule ExUnit.Diff do
   end
 
   defp diff_string_concat(left, nil, indexes, _left_length, right, env) do
-    {parsed_diff, parsed_post_env} = diff_string(left, right, ?", env)
-    left_diff = rebuild_concat_string(parsed_diff.left, nil, indexes)
+    {diff, parsed_post_env} = diff_string(left, right, ?", env)
+    left_diff = rebuild_concat_string(diff.left, nil, indexes)
 
-    diff = %__MODULE__{parsed_diff | left: left_diff}
+    diff = %{diff | left: left_diff}
     {diff, parsed_post_env}
   end
 
@@ -1156,6 +1162,7 @@ defmodule ExUnit.Diff do
     else
       other
       |> Map.to_list()
+      |> Enum.sort()
       |> Enum.map(&escape_pair/1)
       |> build_map_or_struct(struct)
     end
