@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 # Module responsible for fetching (getting/updating)
 # dependencies from their sources.
 #
@@ -51,6 +55,10 @@ defmodule Mix.Dep.Fetcher do
     cond do
       # Dependencies that cannot be fetched are always compiled afterwards
       not scm.fetchable?() ->
+        if not scm.checked_out?(opts) do
+          Mix.shell().error("warning: missing dependency #{format_dep(dep)}")
+        end
+
         {dep, [app | acc], lock}
 
       # If the dependency is not available or we have a lock mismatch
@@ -119,7 +127,7 @@ defmodule Mix.Dep.Fetcher do
 
     for %Mix.Dep{app: app, scm: scm} <- deps, scm.fetchable?() do
       build_path
-      |> Path.join("*/lib/#{app}/.mix/compile.fetch")
+      |> Path.join("*/lib/#{app}/.mix/compile.elixir_scm")
       |> Path.wildcard(match_dot: true)
       |> Enum.each(&File.rm/1)
     end
