@@ -767,7 +767,16 @@ defmodule Mix.Project do
   """
   @spec build_path(keyword) :: Path.t()
   def build_path(config \\ config()) do
-    System.get_env("MIX_BUILD_PATH") || config[:deps_build_path] || do_build_path(config)
+    cond do
+      deps_build_path = config[:deps_build_path] ->
+        deps_build_path
+
+      build_path = System.get_env("MIX_BUILD_PATH") ->
+        Path.expand(build_path)
+
+      true ->
+        do_build_path(config)
+    end
   end
 
   defp do_build_path(config) do
