@@ -1,3 +1,7 @@
+# SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: 2021 The Elixir Team
+# SPDX-FileCopyrightText: 2012 Plataformatec
+
 Code.require_file("../test_helper.exs", __DIR__)
 
 defmodule Mix.TaskTest do
@@ -39,7 +43,7 @@ defmodule Mix.TaskTest do
   end
 
   test "run/2 converts OptionParser.ParseError into Mix errors" do
-    message = "Could not invoke task \"hello\": 1 error found!\n--unknown : Unknown option"
+    message = ~r"Could not invoke task \"hello\": 1 error found!\n--unknown : Unknown option"
 
     assert_raise Mix.Error, message, fn ->
       Mix.Task.run("hello", ["--parser", "--unknown"])
@@ -48,7 +52,7 @@ defmodule Mix.TaskTest do
     Mix.Task.clear()
 
     message =
-      "Could not invoke task \"hello\": 1 error found!\n--int : Expected type integer, got \"foo\""
+      ~r"Could not invoke task \"hello\": 1 error found!\n--int : Expected type integer, got \"foo\""
 
     assert_raise Mix.Error, message, fn ->
       Mix.Task.run("hello", ["--parser", "--int", "foo"])
@@ -59,7 +63,8 @@ defmodule Mix.TaskTest do
     Mix.shell(Mix.Shell.IO)
     Mix.debug(true)
 
-    assert ExUnit.CaptureIO.capture_io(fn -> Mix.Task.run("hello") end) =~ "** Running mix hello"
+    assert ExUnit.CaptureIO.capture_io(fn -> Mix.Task.run("hello") end) =~
+             ~r"-> Running mix hello\n<- Ran mix hello in \d+ms"
   after
     Mix.shell(Mix.Shell.Process)
     Mix.debug(false)
@@ -78,9 +83,9 @@ defmodule Mix.TaskTest do
   end
 
   test "run/2 tries to load deps if task is missing" do
-    Mix.Project.push(DepsApp)
-
     in_fixture("deps_status", fn ->
+      Mix.Project.push(DepsApp)
+
       assert_raise Mix.NoTaskError, fn ->
         Mix.Task.run("task_hello")
       end
@@ -93,7 +98,7 @@ defmodule Mix.TaskTest do
       """)
 
       # Clean up the tasks and update task
-      Mix.TasksServer.clear()
+      Mix.Task.clear()
 
       # Task was found from deps loadpaths
       assert Mix.Task.run("task_hello") == "Hello World v1"
@@ -102,7 +107,7 @@ defmodule Mix.TaskTest do
       assert Mix.Task.run("compile") != :noop
 
       # Clean up the tasks and update task
-      Mix.TasksServer.clear()
+      Mix.Task.clear()
 
       File.write!("custom/raw_repo/lib/task_hello.ex", """
       defmodule Mix.Tasks.TaskHello do
@@ -159,6 +164,58 @@ defmodule Mix.TaskTest do
     end)
   end
 
+  test "aliases considered for umbrella apps" do
+    in_fixture("umbrella_test", fn ->
+      Mix.Project.in_project(:umbrella, ".", fn _ ->
+        File.mkdir_p!("apps/baz/lib/")
+
+        File.write!("apps/baz/mix.exs", """
+        defmodule Baz.MixProject do
+          use Mix.Project
+
+          def project do
+            [
+              app: :baz,
+              version: "0.1.0",
+              aliases: [help: fn _ -> raise "oops" end]
+            ]
+          end
+        end
+        """)
+
+        File.write!("apps/baz/lib/baz.ex", """
+        defmodule Baz do
+          def hello do
+            :world
+          end
+        end
+        """)
+
+        Mix.Task.run("compile")
+        assert_received {:mix_shell, :info, ["==> foo"]}
+        assert_received {:mix_shell, :info, ["==> bar"]}
+        assert_received {:mix_shell, :info, ["==> baz"]}
+        Mix.shell().flush()
+
+        # A child alias does not overlap an umbrella task
+        assert Mix.Task.run("help") == :ok
+
+        # A missing umbrella alias can be found in children
+        assert [:ok, :ok] = Mix.Task.run("mytask")
+        assert_received {:mix_shell, :info, ["==> foo"]}
+        assert_received {:mix_shell, :info, ["foo_running"]}
+        assert_received {:mix_shell, :info, ["==> bar"]}
+        assert_received {:mix_shell, :info, ["bar_running"]}
+        refute_received {:mix_shell, :info, ["==> baz"]}
+
+        # An unknown task or alias anywhere fails
+        assert_raise Mix.NoTaskError, fn ->
+          Mix.Task.run("completely_unknown")
+        end
+      end)
+    end)
+  end
+
   test "reenable/1 for recursive inside umbrella child" do
     in_fixture("umbrella_dep/deps/umbrella", fn ->
       Mix.Project.in_project(:umbrella, ".", fn _ ->
@@ -235,18 +292,6 @@ defmodule Mix.TaskTest do
     assert Mix.Task.moduledoc(Mix.Tasks.Hello) == "A test task.\n"
   end
 
-  test "preferred_cli_env/1 returns nil for missing task" do
-    assert Mix.Task.preferred_cli_env(:no_task) == nil
-  end
-
-  test "preferred_cli_env/1 returns nil when task does not have @preferred_cli_env attribute" do
-    assert Mix.Task.preferred_cli_env(:deps) == nil
-  end
-
-  test "preferred_cli_env/1 returns specified @preferred_cli_env attribute" do
-    assert Mix.Task.preferred_cli_env(:test) == :test
-  end
-
   test "shortdoc/1" do
     assert Mix.Task.shortdoc(Mix.Tasks.Hello) == "This is short documentation, see"
   end
